"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.swaggerToTypeScript = void 0;
const lodash_1 = __importDefault(require("lodash"));
const prettier_1 = __importDefault(require("prettier"));
const swagger2openapi_1 = require("swagger2openapi");
const parse_1 = require("./parse");
const render_1 = require("./render");
async function swaggerToTypeScript(document) {
    const parser = new parse_1.Parser();
    if (document.swagger?.startsWith('2.')) {
        fixMissingRef(document);
        await parser.init((await (0, swagger2openapi_1.convertObj)(document, { resolve: false })).openapi);
    }
    else {
        await parser.init(document);
    }
    const code = [
        ` /* eslint-disable */
      import { toFormData } from 'axios'
      import type { AxiosResponse, AxiosRequestConfig, AxiosInstance } from 'axios'
      type PickData<T> = T extends { data?: any } ? T['data'] : any
      const interpolatePath = (str: string, values: any) => str.replace(/{([^{}]+)}/g, (_, g1) => values[g1])
    `,
    ];
    const ops = parser.getProcessedOperationObjects() || [];
    const tasks = ops.map((el) => () => (0, render_1.renderOperation)(el));
    while (tasks.length) {
        code.push(await tasks.pop()());
    }
    code.push(`
    const collectApis = () => ({
      ${ops
        .reduce((acc, el) => [...acc, `...${el.operationId}(),`], [])
        .join('\n')}
    });

    export type Collection = ReturnType<typeof collectApis>
    export type Keys = keyof Collection
    export type ApiTypes<K extends Keys> = Collection[K][0]
    export type ApiRequestParams<K extends Keys> = Parameters<Collection[K][1]>
    
    export type ApiDataTypes<K extends Keys, T extends keyof ApiTypes<Keys>> = ApiTypes<K>[T]
    
    export function createRequest(axios: AxiosInstance) {
      const collections = collectApis()
    
      return function request<K extends Keys, U = any>(key: K) {
        const fn = collections[key][1] as (...args: any[]) => AxiosRequestConfig;
    
        return (...args: ApiRequestParams<K>) => axios(fn(...args)) as Promise<U>
      }
    }
    
    type Arg<K extends Keys> = ApiRequestParams<K>
    type BaseResp<K extends Keys, U extends "Response" | "ResponseData"> = Promise<AxiosResponse<ApiDataTypes<K, U>>>;

    export interface BaseRequest<T extends "Response" | "ResponseData"> {
      ${ops
        .map((el) => ({
        ...el,
        title: el.description || '',
        req: `"${el.__extra__.method.toUpperCase()} ${el.__extra__.url}"`,
    }))
        .map((el) => `
          /** @summary ${el.summary} */
          <K extends ${el.req}>(key: K): (...args: Arg<K>) => BaseResp<K, T>;
        `)
        .join('\n')}
    }

    export function createAxiosAPI(axios: AxiosInstance) {
      return createRequest(axios) as BaseRequest<'Response'>
    }
  `);
    const fullCode = await prettier_1.default.format(code.join('\n'), {
        parser: 'typescript',
        tabWidth: 2,
        printWidth: 120,
        proseWrap: 'never',
        singleQuote: true,
        semi: false,
        trailingComma: 'es5',
        endOfLine: 'auto',
    });
    return fullCode;
}
exports.swaggerToTypeScript = swaggerToTypeScript;
function fixMissingRef(obj, root = obj) {
    lodash_1.default.forEach(obj, (value) => {
        if (value !== null && typeof value === 'object') {
            // 删除无效引用
            if ('$ref' in value) {
                if (!lodash_1.default.get(root, value.$ref.replace('#/', '').split('/'))) {
                    delete value.$ref;
                    delete value.originalRef;
                }
            }
            else {
                lodash_1.default.forEach(value, (val) => fixMissingRef(val, root));
            }
        }
    });
    return obj;
}
//# sourceMappingURL=index.js.map